// loopTimer_max31856.ino
// example of using the MAX31856_noDelay library and checking the loop time using a loopTimer
/  For two or more Thermocouples see the dual_max31856 example
/* (c)2019 Forward Computing and Control Pty. Ltd., NSW Australia
 * All rights reserved.
 * Subject to the BSD licence below, this code may be freely used for both private and commerical use.
 * Provide this copyright is maintained.
 */
 
#include <MAX31856_noDelay.h>
// LoopTimer_Input.ino
#include <loopTimer.h>
// install the loopTimer library from https://www.forward.com.au/pfod/ArduinoProgramming/RealTimeArduino/TimingDelaysInArduino.html
// loopTimer.h also needs the millisDelay library installed from https://www.forward.com.au/pfod/ArduinoProgramming/TimingDelaysInArduino.html
#include <millisDelay.h>
#include <pfodBufferedStream.h>
// install pfodParser library from https://www.forward.com.au/pfod/pfodParserLibraries/index.html
// to get pfodBufferedStream

const size_t bufSize = 500; // to allow for fault outputs
uint8_t buf[bufSize];
pfodBufferedStream bufferedStream(115200, buf, bufSize, false); // default is blocking == true, if set blocking arg == false, do not block just drop excess bytes
// with blocking true do not miss any output, but may block loop();

// Use software SPI: CS, DI, DO, CLK
MAX31856_noDelay maxthermo = MAX31856_noDelay(10, 11, 12, 13);
// use hardware SPI, just pass in the CS pin
//MAX31856_noDelay maxthermo = MAX31856_noDelay(10);

millisDelay readingsTimer;
unsigned long READINGS_REPEAT_MS = 1000;
bool readingStarted = false; // set to true if reading already running

millisDelay max31856Delay;
unsigned long MAX31856_DELAY_MS = 190; // max 185ms for single shot reading

void setup() {
  Serial.begin(115200);
  for (int i = 10; i > 0; i--) {
    Serial.println(i);
    delay(500);
  }
  Serial.println("MAX31856 thermocouple test");

  maxthermo.begin(); 
  // maxthermo.resetDefaults(); // no need to call this here 
  // the defaults at the top of MAX31856_noDelay.cpp are set on the first call to any on of the library methods if resetDefaults() not called here
  // this lets you initialise the CS lines for other SPI modules before writing to this module

  maxthermo.setThermocoupleType(MAX31856_TCTYPE_K); // automatically calls set defaults first.

  Serial.print("Thermocouple type: ");
  switch (maxthermo.getThermocoupleType() ) {
  case MAX31856_TCTYPE_B:
    Serial.println("B Type");
    break;
  case MAX31856_TCTYPE_E:
    Serial.println("E Type");
    break;
  case MAX31856_TCTYPE_J:
    Serial.println("J Type");
    break;
  case MAX31856_TCTYPE_K:
    Serial.println("K Type");
    break;
  case MAX31856_TCTYPE_N:
    Serial.println("N Type");
    break;
  case MAX31856_TCTYPE_R:
    Serial.println("R Type");
    break;
  case MAX31856_TCTYPE_S:
    Serial.println("S Type");
    break;
  case MAX31856_TCTYPE_T:
    Serial.println("T Type");
    break;
  case MAX31856_VMODE_G8:
    Serial.println("Voltage x8 Gain mode");
    break;
  case MAX31856_VMODE_G32:
    Serial.println("Voltage x32 Gain mode");
    break;
  default:
    Serial.println("Unknown");
    break;
  }

  bufferedStream.connect(&Serial);  // connect this after any setup print statements 
  readingsTimer.start(READINGS_REPEAT_MS);

}

// only on Thermocouple read here
void loop() {
  bufferedStream.available(); // this tickles the bufferedStream to release the next char
  loopTimer.check(&bufferedStream);

  if (readingsTimer.justFinished()) {
    readingsTimer.repeat();
    if (!readingStarted) {
      maxthermo.oneShotTemperature();       // start one now
      max31856Delay.start(MAX31856_DELAY_MS); // start delay to pick up results
    }
  }

  if (max31856Delay.justFinished()) {
    readingStarted = false;
    // can pick up both results now
    bufferedStream.print("Cold Junction Temp: ");
    bufferedStream.println(maxthermo.readCJTemperature());

    bufferedStream.print("Thermocouple Temp: ");
    bufferedStream.println(maxthermo.readThermocoupleTemperature());
    // Check and print any faults
    uint8_t fault = maxthermo.readFault();
    if (fault) {
      // lots of faults here since SPI not connected
      bufferedStream.println(" Thermocouple Fault");
      //      if (fault & MAX31856_FAULT_CJRANGE) bufferedStream.println("Cold Junction Range Fault");
      //      if (fault & MAX31856_FAULT_TCRANGE) bufferedStream.println("Thermocouple Range Fault");
      //      if (fault & MAX31856_FAULT_CJHIGH)  bufferedStream.println("Cold Junction High Fault");
      //      if (fault & MAX31856_FAULT_CJLOW)   bufferedStream.println("Cold Junction Low Fault");
      //      if (fault & MAX31856_FAULT_TCHIGH)  bufferedStream.println("Thermocouple High Fault");
      //      if (fault & MAX31856_FAULT_TCLOW)   bufferedStream.println("Thermocouple Low Fault");
      //      if (fault & MAX31856_FAULT_OVUV)    bufferedStream.println("Over/Under Voltage Fault");
      //      if (fault & MAX31856_FAULT_OPEN)    bufferedStream.println("Thermocouple Open Fault");
    }
  }
}
